/*
 * main.c
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

/* Sizes of buffers malloced and then freed in turn.
 * Can be used to investigate at which point _sbrk() is called (by setting a breakpoint).
 *
 * Note: The first call to printf() calls malloc_r() to allocate a 512 byte (BUFSIZ) line buffer.
 *       If the malloc_r calls fails, printf calls __sbprintf() to output the text.
 *       __sbprintf allocates a 512 byte buffer on the stack, which allows printf to generate output
 *       when malloc fails but at the expense of a larger stack requirement. */
#define NUM_MALLOC_SIZES 10
const size_t malloc_sizes[NUM_MALLOC_SIZES] = {256, 80, 128, 512, 368, 1024, 100, 512, 2048, 1024};

int main(void) {
    unsigned int size_index;
    char *text_buffer;

    for (size_index = 0; size_index < NUM_MALLOC_SIZES; size_index++)
    {
        const size_t buffer_size = malloc_sizes[size_index];

        errno = 0;
        text_buffer = malloc (buffer_size);
        if (text_buffer != NULL)
        {
            sprintf (text_buffer, "This is a text string in buffer of size %u malloced at %p\n", buffer_size, text_buffer);
            printf ("%s", text_buffer);
            free (text_buffer);
        }
        else
        {
            printf ("Malloc(%u) failed errno=%d (%s)\n", buffer_size, errno, strerror (errno));
        }
    }

    return 0;
}
